/**
  *  \file client/dlg-groundcombat.cc
  *  \brief Ground Combat Prediction Dialog
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <algorithm>
#include "client/dlg-groundcombat.h"
#include "cpluslib/format.h"
#include "game/config.h"
#include "game/formula.h"
#include "game/host.h"
#include "game/planet.h"
#include "game/racename.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/richdocument.h"
#include "ui/skin.h"
#include "ui/ui.h"
#include "ui/uiglobal.h"
#include "ui/widgets/button.h"
#include "ui/widgets/keytranslator.h"
#include "ui/window.h"
#include "util/misc.h"
#include "util/nls.h"

/** Display Ground Combat Prediction dialog for own planet.
    \param planet Planet to display dialog for. Must be owned by player, i.e. have full info
    \param defense Assume this many defense posts */
void
doGroundCombatPrediction(const GPlanet& planet, int defense)
{
    ObjectHolder h;

    UIWindow& win = h.add(new UIWindow(_("Ground Combat"), UIWindow::BLUE_WINDOW, COLOR_GRAY));
    win.setLayout(UIVBoxLayout::instance5);

    // Find name column width
    int nameWidth = 0;
    for (int i = 1; i <= NUM_PLAYERS; ++i) {
        nameWidth = std::max(fonts[FONT_NORMAL]->getTextWidth(player_racenames.getFullName(i)), nameWidth);
    }

    // Find clan column width
    const char* defense_fmt = _("(defense) %d");
    const char* header_text = _("Clans needed to win");
    int ccWidth = std::max(fonts[FONT_NORMAL]->getTextWidth(format(defense_fmt, 999999)),
                           fonts[FONT_NORMAL]->getTextWidth(header_text));
    int totalWidth = nameWidth + ccWidth;

    // Build document
    UIRichDocument d(GfxPoint(totalWidth, 10), 0);
    RichDocument& doc = d.getDocument();
    doc.setPageWidth(totalWidth);
    doc.add(RichText(_("Attacker")).withAttribute(RichTextStyleAttribute::Underline));
    doc.addRight(totalWidth, RichText(header_text).withAttribute(RichTextStyleAttribute::Underline));
    doc.addNewline();

    // Compute my strength
    int32_t myStrength;
    if (host.isPHost()) {
        // PS := Colonists * (20 * GDEFENSE + dp) \ 20
        // i.e. for 0 dp planet = Colonists * GDEFENSE
        //         20 dp planet = Colonists * (GDEFENSE+1)
        myStrength = (planet.getColonists() *
                      (20 * config.GroundDefenseFactor(planet.getOwner()) + defense))
            / 20;
    } else {
        // PS := ERnd( Colonists * GDEFENSE * (Defense + 20) / 20 )
        // i.e. for 0 dp planet = Colonists * GDEFENSE
        //         20 dp planet = Colonists * GDEFENSE*2 }
        myStrength = rdivaddl(planet.getColonists() * config.GroundDefenseFactor(planet.getOwner()) * (defense + 20),
                              20, 0);
    }

    // Output
    for (int i = 1; i <= NUM_PLAYERS; ++i) {
        if (i == planet.getOwner()) {
            doc.add(RichText(UIColor::tc_Green, player_racenames.getFullName(i)));
            doc.addRight(totalWidth, RichText(UIColor::tc_Green, string_t(format(defense_fmt, planet.getColonists()))));
        } else {
            doc.add(player_racenames.getFullName(i));

            // given N=number of attacking clans, their strength is
            //    N * GATTACK
            // which must be >= myStrength for the attack to kill our planet.
            //    N * GATTACK >= myStrength
            // -> N           >= myStrength / GATTACK
            // the minimum N hence is ceil(myStrength / GATTACK)
            int32_t theirRatio = config.GroundKillFactor(i);
            if (theirRatio > 0) {
                int32_t equivClans = (myStrength + (theirRatio-1)) / theirRatio;
                doc.addRight(totalWidth, numToString(equivClans));
            }
        }
        doc.addNewline();
    }

    d.adjustToDocumentSize();
    win.add(d);

    UIGroup& group = h.add(new UIGroup(UIHBoxLayout::instance5));
    group.add(h.add(new UISpacer()));
    group.add(h.add(new UIButton(1, _("OK"), SDLK_RETURN)).addStop());
    group.add(h.add(new UISpacer()));
    win.add(group);
    win.add(h.add(new UIKeyTranslator())
            .add(' ', 1, true)
            .add(SDLK_ESCAPE, 1, true));

    win.pack();
    ui_root->centerWidget(win);
    ui_root->runModal(win);
}
