/**
  *  \file game/reset.cc
  *  \brief GReset: Resetting Stuff
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <algorithm>
#include "game/reset.h"
#include "game/global.h"
#include "game/univ.h"
#include "game/ship.h"
#include "game/planet.h"


namespace {
    bool isReadyList(const GObjectList& list, int mode)
    {
        for (GObjectList::size_type i = 0, n = list.size(); i != n; ++i) {
            if (GShip* sh = dynamic_cast<GShip*>(&*list[i])) {
                // Ship
                if ((mode & GReset::ResetCargo) != 0) {
                    if (sh->cargo_arbiter.isInUse() || sh->isTransporterInUse(GShip::t_Unload) || sh->isTransporterInUse(GShip::t_Transfer)) {
                        return false;
                    }
                }
            } else if (GPlanet* pl = dynamic_cast<GPlanet*>(&*list[i])) {
                // Planet
                if ((mode & GReset::ResetCargo) != 0) {
                    if (pl->cargo_arbiter.isInUse()) {
                        return false;
                    }
                }
            } else {
                // What? Refuse, we don't know what to do with it.
                return false;
            }
        }
        return true;
    }
}


GReset::GReset()
    : mode(ResetMissions | ResetCargo),
      dat_objects(),
      dis_objects()
{ }

GReset::~GReset()
{ }

/** Add location to things-to-reset list.
    \param trn Turn to add
    \param pt Location to add */
void
GReset::addLocation(GGameTurn& trn, GPoint pt)
{
    // Prepare
    GUniverse& dat = trn.getCurrentUniverse();
    GUniverse& dis = trn.getPreviousUniverse();
    GPlayerSet players;
    pt = pt.getCanonicalLocation();

    // Planet
    if (int pid = dat.getPlanetAt(pt)) {
        if (dat.ty_played_planets.isValidIndex(pid)) {
            dat_objects.addObject(dat.ty_played_planets, pid);
            dis_objects.addObject(dis.ty_played_planets, pid);
            players += dat.getPlanet(pid).getOwner();
        }
    }

    // Ships
    for (int sid = dat.ty_played_ships.findNextIndex(0); sid != 0; sid = dat.ty_played_ships.findNextIndex(sid)) {
        if (dat.getShip(sid).getPos() == pt) {
            dat_objects.addObject(dat.ty_played_ships, sid);
            dis_objects.addObject(dis.ty_played_ships, sid);
            players += dat.getShip(sid).getOwner();
        }
    }

    // Remember the command containers
    for (int pl = 1; pl <= NUM_PLAYERS; ++pl) {
        if (players.contains(pl)) {
            GCommandContainer* pcc = &trn.getCommands(pl);
            if (std::find(commands.begin(), commands.end(), pcc) == commands.end()) {
                commands.push_back(pcc);
            }
        }
    }
}

/** Check whether reset operation can be performed.
    A running cargo transfer can prevent resetting. */
bool
GReset::isReady() const
{
    return mode != 0
        && isReadyList(dat_objects, mode)
        && isReadyList(dis_objects, mode);
}

/** Perform the operation. */
void
GReset::commit()
{
    // Self-defense.
    if (!isReady()) {
        throw GError(GError::ePerm, GError::ePerm);
    }

    // Do it
    for (GObjectList::size_type i = 0, n = dat_objects.size(); i != n; ++i) {
        GObject& dat = *dat_objects[i];
        GObject& dis = *dis_objects[i];
        if (GShip* dat_ship = dynamic_cast<GShip*>(&dat)) {
            const GShip* dis_ship = dynamic_cast<GShip*>(&dis);
            ASSERT(dis_ship);

            // Ship record:
            // Keep:    Id, Player               4 bytes
            // Mission: Waypoint, Speed, FC      9 bytes
            // Keep:    Location, Equipment     16 bytes
            // Cargo:   Ammo                     2 bytes
            // Keep:    Torp launcher            2 bytes
            // Mission: Mission, PE, TowID       6 bytes
            // Keep:    Damage, Crew             4 bytes
            // Cargo:   Colonists                2 bytes
            // Mission: Name                    20 bytes
            // Cargo:   Cargo, Transfers        38 bytes
            // Mission: Intercept ID             2 bytes
            // Cargo:   Money                    2 bytes
            //                              -> 107 bytes

            if ((mode & ResetMissions) != 0) {
                // Waypoint, Speed, FC
                dat_ship->setWaypoint(dis_ship->getWaypoint());
                dat_ship->setWarp(dis_ship->getWarp());
                dat_ship->setFCode(dis_ship->getFCode());

                // Mission, PE, TowID + IntID
                dat_ship->setMission(dis_ship->getMission(), dis_ship->getInterceptId(), dis_ship->getTowId());
                dat_ship->setPrimaryEnemy(dis_ship->getPrimaryEnemy());

                // Name
                dat_ship->setName(dis_ship->getName(GObject::PlainName));

                // Commands
                removeCommands(dat_ship->getId(), true);
            }

            if ((mode & ResetCargo) != 0) {
                // Transfers
                dat_ship->getTransporter(GShip::t_Transfer) = dis_ship->getTransporter(GShip::t_Transfer);
                dat_ship->getTransporter(GShip::t_Unload) = dis_ship->getTransporter(GShip::t_Unload);

                // Remaining cargo
                for (GCargoType el = el_FIRST; el < el_COUNT; ++el) {
                    mp16_t dat_cargo = dat_ship->getCargoRaw(el);
                    mp16_t dis_cargo = dis_ship->getCargoRaw(el);
                    if (dat_cargo.isKnown() && dis_cargo.isKnown() && dat_cargo != dis_cargo) {
                        dat_ship->changeCargoRaw(el, dis_cargo - dat_cargo);
                    }
                }
            }
        } else if (GPlanet* dat_planet = dynamic_cast<GPlanet*>(&dat)) {
            const GPlanet* dis_planet = dynamic_cast<GPlanet*>(&dis);
            ASSERT(dis_planet);

            // Planet record:
            // Keep:    PID, Player              4 bytes
            // Mission: FC                       3 bytes
            // Cargo:   Structures               6 bytes
            // Cargo:   N/T/D/M, Col, Sup, MC   28 bytes
            // Keep:    Ground N/T/D/M, Density 24 bytes
            // Mission: Taxes                    4 bytes
            // Keep:    Happy, Natives, Temp    14 bytes
            // Cargo:   Base                     2 bytes
            //                               -> 85 bytes

            // Starbase record
            // Keep:    PID, Player              4 bytes
            // Cargo:   Defense                  2 bytes
            // Keep:    Damage                   2 bytes
            // Cargo:   Tech                     8 bytes
            // Cargo:   Engine storage          18 bytes
            // Cargo:   Hull storage            40 bytes
            // Cargo:   Beam, Torp, Tube store  60 bytes
            // Cargo:   Fighter storage          2 bytes
            // Mission: Recycle, Mission         6 bytes
            // Cargo:   Build order             14 bytes
            //                              -> 156 bytes
            if ((mode & ResetMissions) != 0) {
                // Planet
                dat_planet->setFCode(dis_planet->getFCode());
                dat_planet->setColonistTax(dis_planet->getColonistTax());
                dat_planet->setNativeTax(dis_planet->getNativeTax());

                // Base
                if (dat_planet->hasBase()) {
                    dat_planet->setBaseShipyardOrder(dis_planet->getBaseShipyardOrder(), dis_planet->getBaseShipyardId());
                    dat_planet->setBaseMission(dis_planet->getBaseMission());
                }

                // Commands
                removeCommands(dat_planet->getId(), false);
            }

            if ((mode & ResetCargo) != 0) {
                // Structures
                for (int i = str_Mines; i <= str_BaseDefense; ++i) {
                    dat_planet->setStructures(TPlanetaryStructure(i), dis_planet->getStructures(TPlanetaryStructure(i)));
                }

                // Cargo (N/T/D/M/Sup/MC/Col, Fighters, Torp storage)
                for (GCargoType el = el_FIRST; el < el_COUNT; ++el) {
                    mp32_t dat_cargo = dat_planet->getCargoRaw(el);
                    mp32_t dis_cargo = dis_planet->getCargoRaw(el);
                    if (dat_cargo.isKnown() && dis_cargo.isKnown() && dat_cargo != dis_cargo) {
                        dat_planet->setCargoRaw(el, dis_cargo);
                    }
                }

                // Base build order
                dat_planet->setBuildBaseFlag(dis_planet->isBuildingBase());

                // Base properties
                if (dat_planet->hasBase()) {
                    // Ship build order
                    dat_planet->setBaseBuildOrder(dis_planet->getBaseBuildOrder());

                    // Tech
                    for (int i = tl_Engine; i <= tl_Torpedo; ++i) {
                        dat_planet->setBaseTechLevel(TTechLevel(i), dis_planet->getBaseTechLevel(TTechLevel(i)));
                    }

                    // Base storage
                    for (int i = 1; i <= NUM_ENGINES; ++i) {
                        dat_planet->setBaseEngineStore(i, dis_planet->getBaseEngineStore(i));
                    }
                    for (int i = 1; i <= NUM_HULLS_PER_PLAYER; ++i) {
                        dat_planet->setBaseHullStoreSlot(i, dis_planet->getBaseHullStoreSlot(i));
                    }
                    for (int i = 1; i <= NUM_BEAMS; ++i) {
                        dat_planet->setBaseBeamStore(i, dis_planet->getBaseBeamStore(i));
                    }
                    for (int i = 1; i <= NUM_TORPS; ++i) {
                        dat_planet->setBaseLauncherStore(i, dis_planet->getBaseLauncherStore(i));
                    }
                }
            }
        } else {
            ASSERT(0);
        }
    }
}

void
GReset::removeCommands(int id, bool ship)
{
    for (std::vector<GCommandContainer*>::iterator vi = commands.begin(), ve = commands.end(); vi != ve; ++vi) {
        GCommandContainer& cont = **vi;
        GCommandContainer::iterator iter = cont.begin(), e = cont.end();
        while (iter != e) {
            bool match = (ship
                          ? (*iter)->getAffectedShip() == id
                          : (*iter)->getAffectedPlanet() == id);
            if (match) {
                iter = cont.erase(iter);
            } else {
                ++iter;
            }
        }
    }
}
