/**
  *  \file game/shipquery.h
  *  \brief Ship Query
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_SHIPQUERY_H
#define PCC_V2_GAME_SHIPQUERY_H

#include "game/exp.h"
#include "game/playerset.h"

class GUniverse;
class GHullFunctionList;

/** Ship query. This class contains information required to query for ship properties,
    such as
    - hull functions
    - hull specification
    - weapon effects

    It is a union of the PCC 1.x structures THullFuncSelection and TShipInformation. */
class GShipQuery {
    int hull_id;                      ///< Hull Id. Zero if not known.
    int ship_id;                      ///< Ship Id. Zero if not known.

    GExpLevelSet filter_level_set;    ///< Experience levels for filtering. Display only those that match this filter.
    GExpLevelSet display_level_set;   ///< Experience levels for display. Those matching this filter are shown as active.

    GPlayerSet filter_player_set;     ///< Owner mask for filtering. Display only those that match this filter.
    GPlayerSet display_player_set;    ///< Owner mask for display. Those matching this filter are shown as active.

    int engine_id;                    ///< Engine number. Zero if not known.
    int combat_mass;                  ///< Combat mass. Zero if not known.
    int crew;                         ///< Crew. Zero if not known.
    int owner;                        ///< Ship owner. Zero if not known.
    int used_esb;                     ///< Used Engine-Shield bonus. Set if combat_mass includes ESB.

 public:
    GShipQuery();

    void clear();
    void initForExistingShip(const GUniverse& univ, int sid);
    void complete(const GUniverse& univ);

    void enumShipSpecials(const GUniverse& univ, GHullFunctionList& list, bool includeRacialAbilities) const;

    bool operator==(const GShipQuery& other) const;
    bool operator!=(const GShipQuery& other) const;


    int          getHullId() const;
    void         setHullId(int id);

    int          getShipId() const;
    void         setShipId(int id);

    GExpLevelSet getLevelFilterSet() const;
    void         setLevelFilterSet(GExpLevelSet set);

    GExpLevelSet getLevelDisplaySet() const;
    void         setLevelDisplaySet(GExpLevelSet set);

    GPlayerSet   getPlayerFilterSet() const;
    void         setPlayerFilterSet(GPlayerSet set);

    GPlayerSet   getPlayerDisplaySet() const;
    void         setPlayerDisplaySet(GPlayerSet set);

    int          getEngineId() const;
    void         setEngineId(int id);

    int          getCombatMass() const;
    int          getUsedESB() const;
    void         setCombatMass(int mass, int usedESB);

    int          getCrew() const;
    void         setCrew(int id);

    int          getOwner() const;
    void         setOwner(int id);
};

/** Compare for inequality. */
inline bool
GShipQuery::operator!=(const GShipQuery& other) const
{
    return !operator==(other);
}

/** Get hull Id. \return hull Id, 0 if not known. */
inline int
GShipQuery::getHullId() const
{
    return hull_id;
}

/** Set hull Id. \param id Hull Id, 0 if not known. */
inline void
GShipQuery::setHullId(int id)
{
    this->hull_id = id;
}

/** Get ship Id. \return ship Id, 0 if not known. */
inline int
GShipQuery::getShipId() const
{
    return ship_id;
}

/** Set ship Id. \param id Ship Id, 0 if not known. */
inline void
GShipQuery::setShipId(int id)
{
    this->ship_id = id;
}

inline GExpLevelSet
GShipQuery::getLevelFilterSet() const
{
    return filter_level_set;
}

inline void
GShipQuery::setLevelFilterSet(GExpLevelSet set)
{
    this->filter_level_set = set;
}

inline GExpLevelSet
GShipQuery::getLevelDisplaySet() const
{
    return display_level_set;
}

inline void
GShipQuery::setLevelDisplaySet(GExpLevelSet set)
{
    this->display_level_set = set;
}

inline GPlayerSet
GShipQuery::getPlayerFilterSet() const
{
    return filter_player_set;
}

inline void
GShipQuery::setPlayerFilterSet(GPlayerSet set)
{
    this->filter_player_set = set;
}

inline GPlayerSet
GShipQuery::getPlayerDisplaySet() const
{
    return display_player_set;
}

inline void
GShipQuery::setPlayerDisplaySet(GPlayerSet set)
{
    this->display_player_set = set;
}

inline int
GShipQuery::getEngineId() const
{
    return engine_id;
}

inline void
GShipQuery::setEngineId(int id)
{
    this->engine_id = id;
}

inline int
GShipQuery::getCombatMass() const
{
    return combat_mass;
}

inline int
GShipQuery::getUsedESB() const
{
    return used_esb;
}

inline void
GShipQuery::setCombatMass(int mass, int usedESB)
{
    this->combat_mass = mass;
    this->used_esb = used_esb;
}

inline int
GShipQuery::getCrew() const
{
    return crew;
}

inline void
GShipQuery::setCrew(int id)
{
    this->crew = id;
}

inline int
GShipQuery::getOwner() const
{
    return owner;
}

inline void
GShipQuery::setOwner(int id)
{
    this->owner = id;
}

#endif
