/**
  *  \file int/structure.cc
  *  \brief Interpreter: Structures
  *
  *  A structure type has a fixed name/value-slot mapping for all objects
  *  of its type (unlike a hash, which has a possibly-different mapping
  *  for each object). We therefore have two objects, the structure type
  *  (IntStructureTypeData) for the mapping, and the actual instance value
  *  (IntStructureValueData). As usual, we have two C++ types for each,
  *  the actual "heavy-weight" data object, and the "light-weight"
  *  interpreter object.
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/structure.h"
#include "int/interror.h"
#include "int/propenum.h"
#include "int/vmio.h"

/************************** IntStructureTypeData *************************/

IntStructureTypeData::IntStructureTypeData()
{ }

IntStructureTypeData::~IntStructureTypeData()
{ }

/**************************** IntStructureType ***************************/

IntStructureType::IntStructureType(Ptr<IntStructureTypeData> type)
    : type(type)
{ }

IntStructureType::~IntStructureType()
{ }

// IntValue:
string_t
IntStructureType::toString(bool /*readable*/)
{
    return "#<struct-type>";
}

void
IntStructureType::store(IntTagNode& sv, Stream& /*aux*/)
{
    IntVMSaveContext* vsc = IntVMSaveContext::getCurrentInstance();
    if (vsc != 0) {
        sv.tag   = IntTagNode::Tag_StructType;
        sv.value = vsc->addStructureType(*type);
    } else {
        throw IntError::notSerializable();
    }
}

IntStructureType*
IntStructureType::clone()
{
    return new IntStructureType(type);
}

/************************* IntStructureValueData *************************/

IntStructureValueData::IntStructureValueData(Ptr<IntStructureTypeData> type)
    : type(type),
      data()
{ }

IntStructureValueData::~IntStructureValueData()
{ }

/*************************** IntStructureValue ***************************/

IntStructureValue::IntStructureValue(Ptr<IntStructureValueData> value)
    : value(value)
{ }

IntStructureValue::~IntStructureValue()
{ }

// IntValue:
string_t
IntStructureValue::toString(bool /*readable*/)
{
    return "#<struct>";
}

void
IntStructureValue::store(IntTagNode& sv, Stream& /*aux*/)
{
    IntVMSaveContext* vsc = IntVMSaveContext::getCurrentInstance();
    if (vsc != 0) {
        sv.tag   = IntTagNode::Tag_Struct;
        sv.value = vsc->addStructureValue(*value);
    } else {
        throw IntError::notSerializable();
    }
}

IntStructureValue*
IntStructureValue::clone()
{
    return new IntStructureValue(value);
}

// IntContext:
int32_t
IntStructureValue::lookup(const string_t& name)
{
    return value->type->names.lookup(name);
}

void
IntStructureValue::set(int32_t index, IntValue* value)
{
    this->value->data.set(index, value);
}

IntValue*
IntStructureValue::get(int32_t index)
{
    return IntValue::cloneOf(value->data[index]);
}

GObject*
IntStructureValue::getObject()
{
    return 0;
}

void
IntStructureValue::enumProperties(IntPropertyAcceptor& acceptor)
{
    acceptor.enumNames(value->type->names);
}
